@@ -1,7 +1,7 @@
 package Carmel;
 use strict;
 use 5.012000;
-use version; our $VERSION = version->declare('v0.1.40');
+use version; our $VERSION = version->declare('v0.1.41');
 
 1;
 __END__
@@ -29,7 +29,10 @@ Carmel - CPAN Artifact Repository Manager
   # show a location where a module is installed
   carmel show Plack
 
-  # update snapshot
+  # update Plack to the latest
+  carmel update Plack
+
+  # update all the modules in the snapshot
   carmel update
 
   # Runs your perl script with modules from artifacts
