@@ -97,31 +97,59 @@ sub cmd_inject {
 sub cmd_update {
     my($self, @args) = @_;
 
-    die "Usage: carmel update\n" if @args; # TODO supprot args
+    my $current = $self->snapshot
+      or die "Can't run carmel update without snapshot. Run `carmel install` first.\n";
 
-    # reinstall everything in cpanfile to build artifacts
-    my $cpanfile = $self->try_cpanfile
-      or die "Can't locate 'cpanfile' to load module list.\n";
-    $self->install_with_cpanfile(Module::CPANfile->load($cpanfile));
+    for my $module (@args) {
+        my $dist = $current->find($module)
+          or die "$module is not found in the snapshot.\n";
+    }
 
-    # then rebuild the snapshot
-    my @artifacts = $self->install_from_cpanfile(1);
+    # create a new snapshot and copy needed dists over, since Carton::Snapshot is immutable for now
+    my $snapshot = Carton::Snapshot->new(path => $current->path);
+
+    if (@args) {
+        my @dists = grep !$self->dist_provides_any($_, \@args), $current->distributions;
+        for my $dist (@dists) {
+            $snapshot->add_distribution($dist);
+        }
+        $self->install_with_snapshot($snapshot, [], @args);
+    } else {
+        # remove everything from the snapshot
+        my $cpanfile = $self->try_cpanfile
+          or die "Can't locate 'cpanfile' to load module list.\n";
+        $self->install_with_cpanfile(Module::CPANfile->load($cpanfile), $snapshot);
+    }
+
+    # rebuild the snapshot
+    my @artifacts = $self->install_from_cpanfile($snapshot);
     $self->dump_bootstrap(\@artifacts);
     $self->save_snapshot(\@artifacts);
 }
 
+sub dist_provides_any {
+    my($self, $dist, $modules) = @_;
+
+    my $provides = $dist->provides or return;
+    for my $mod (@$modules) {
+        return 1 if $provides->{$mod};
+    }
+
+    return;
+}
+
 sub cmd_install {
     my($self, @args) = @_;
 
     die "Usage: carmel install\n" if @args;
 
-    my @artifacts = $self->install_from_cpanfile;
+    my @artifacts = $self->install_from_cpanfile($self->snapshot);
     $self->dump_bootstrap(\@artifacts);
     $self->save_snapshot(\@artifacts);
 }
 
 sub install_from_cpanfile {
-    my($self, $no_snapshot) = @_;
+    my($self, $snapshot) = @_;
 
     my $requirements = CPAN::Meta::Requirements->new;
     $self->resolve(
@@ -134,7 +162,7 @@ sub install_from_cpanfile {
             $requirements->add_string_requirement($module => $want_version);
         },
         1, # strict
-        $no_snapshot,
+        $snapshot,
     );
 
     if (my @missing = $requirements->required_modules) {
@@ -144,11 +172,11 @@ sub install_from_cpanfile {
             },
         });
         print "---> Installing new dependencies: ", join(", ", @missing), "\n";
-        $self->install_with_cpanfile($cpanfile, $no_snapshot ? () : $self->snapshot );
+        $self->install_with_cpanfile($cpanfile, $snapshot);
     }
 
     my @artifacts;
-    $self->resolve(sub { push @artifacts, $_[0] }, undef, 0, $no_snapshot);
+    $self->resolve(sub { push @artifacts, $_[0] }, undef, 0, $snapshot);
 
     # $self->requirements has been upgraded at this point with the whole subreqs
     printf "---> Complete! %d cpanfile dependencies. %d modules installed.\n" .
@@ -172,16 +200,22 @@ sub install_with_cpanfile {
 
     my @options = ("--cpanfile", $path);
 
+    $self->install_with_snapshot($snapshot, [ "--cpanfile", $path, "--installdeps" ], ".");
+}
+
+sub install_with_snapshot {
+    my($self, $snapshot, $options, @cmd) = @_;
+
     if ($snapshot) {
         my $path = Path::Tiny->tempfile;
         $snapshot->write_index($path);
-        push @options,
+        push @$options,
           "--mirror-index", $path,
           "--cascade-search",
           "--mirror", "http://cpan.metacpan.org";
     }
 
-    $self->install("--installdeps", @options, ".");
+    $self->install(@$options, @cmd);
 }
 
 sub install {
@@ -624,14 +658,14 @@ sub resolve_recursive {
 }
 
 sub resolve {
-    my($self, $cb, $missing_cb, $strict, $no_snapshot) = @_;
+    my($self, $cb, $missing_cb, $strict, $snapshot) = @_;
     $missing_cb ||= sub {
         my($module, $want_version, $dist, $depth) = @_;
         die "Can't find an artifact for $module => $want_version\n" .
             "You need to run `carmel install` first to get the modules installed and artifacts built.\n";
     };
 
-    my $snapshot = $no_snapshot ? undef : $self->snapshot;
+    $snapshot ||= $self->snapshot;
 
     $self->resolve_recursive($self->requirements, $self->requirements->clone, $snapshot,
                              {}, $cb, $missing_cb, $strict, 0);
